/*
 * Logic.java
 *
 * Created on July 31, 2006, 11:47 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package ee5travel.logic;

import ee5travel.entities.Person;
import ee5travel.entities.Trip;
import ee5travel.entities.Triptype;
import java.util.Collection;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;

/**
 * <p>Class containing the business logic for this application.  This is
 * separated so that it can be used separately, or in conjunction with
 * an EJB or web service that wraps it.</p>
 */
public class Logic {
    

    // ------------------------------------------------------------ Constructors


    /**
     * <p>Construct a default instance.</p>
     */
    public Logic() {
        ;
    }


    /**
     * <p>Construct a fully configured instance.</p>
     *
     * @param emf The entity manager factory to be used
     */
    public Logic(EntityManagerFactory emf) {
        this.entityManagerFactory = emf;
    }


    // ------------------------------------------------------- Public Properties


    /**
     * <p>The entity manager factory we will use for accessing the database.
     * <strong>WARNING</strong>:  resource injection will only occur
     * if this is a container managed object (such as a JSF managed
     * bean), so provide getter/setter methods as well to allow
     * manual injection.</p>
     */
    @PersistenceUnit(unitName="travel")
    EntityManagerFactory entityManagerFactory;


    /**
     * <p>Return the entity manager factory used for accessing the database.</p>
     */
    public EntityManagerFactory getEntityManagerFactory() {
        return this.entityManagerFactory;
    }


    /**
     * <p>Set the entity manager factory used for accessing the database.</p>
     *
     * @param entityManagerFactory The new entity manager factory
     */
    public void setEntityManagerFactory
      (EntityManagerFactory entityManagerFactory) {
        this.entityManagerFactory = entityManagerFactory;
    }


    /**
     * <p>Read-only property to return all {@link Person} instances.
     * You would not usually see such a method in a real application,
     * but we know that our demo database has a very constrained
     * set of rows.</p>
     */
    public List<Person> getPersons() {

        EntityManager em = entityManager();
        try {
            Query query = em.createQuery("SELECT p FROM Person p");
            return query.getResultList();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Read-only property to return all {@link Triptype} instances.
     */
    public List<Triptype> getTriptypes() {

        EntityManager em = entityManager();
        try {
            Query query = em.createQuery("SELECT t FROM Triptype t");
            return query.getResultList();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            em.close();
        }

    }


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Add a new {@link Person} instance.</p>
     *
     * @param person The {@link Person} instance to be added
     *
     * @return The updated {@link Person} instance
     */
    public Person addPerson(Person person) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            em.persist(person);
            et.commit();
            return person;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }

    }


    /**
     * <p>Add a new {@link Trip} for the specified {@link Person}.</p>
     *
     * @param person The {@link Person} for whom to add a new {@link Trip}
     * @param trip The {@link Trip} to be added
     *
     * @return The updated {@link Trip} instance
     */
    public Trip addTrip(Person person, Trip trip) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            person = em.find(Person.class, person.getPersonid());
            trip.setPersonid(person);
            em.persist(trip);
            person.getTripCollection().add(trip);
            et.commit();
            return trip;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }

    }


    /**
     * <p>Find an existing {@link Person} instance by id.</p>
     *
     * @return The found {@link Person} instance, if any;
     *  otherwise <code>null</code>
     */
    public Person findPersonById(int id) {

        EntityManager em = entityManager();
        try {
            return em.find(Person.class, id);
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Find an existing {@link Trip} instance by id.</p>
     *
     * @return The found {@link Trip} instance, if any;
     *  otherwise <code>null</code>
     */
    public Trip findTripById(int id) {

        EntityManager em = entityManager();
        try {
            return em.find(Trip.class, id);
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Return a list of {@link Trip}s for the specified
     * {@link Person}.</p>
     *
     * @param person {@link Person} for which to find trips
     */
    public List<Trip> findTripsByPerson(Person person) {

        EntityManager em = entityManager();
        try {
            Query query = em.createQuery("SELECT t FROM Trip t WHERE t.personid = :personid");
            query.setParameter("personid", person.getPersonid());
            return query.getResultList();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Find an existing {@link Triptype} instance by id.</p>
     *
     * @return The found {@link Triptype} instance, if any;
     *  otherwise <code>null</code>
     */
    public Triptype findTriptypeById(int id) {

        EntityManager em = entityManager();
        try {
            return em.find(Triptype.class, id);
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            em.close();
        }

    }


    /**
     * <p>Remove an existing {@link Person} instance.</p>
     *
     * @param person The {@link Person} instance to be removed
     */
    public void removePerson(Person person) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            person = em.merge(person);
            em.remove(person);
            et.commit();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }

    }


    /**
     * <p>Remove an existing {@link Trip} instance.</p>
     *
     * @param trip The {@link Trip} instance to be removed
     */
    public void removeTrip(Trip trip) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            trip = em.merge(trip);
            Person person = em.find(Person.class, trip.getPersonid().getPersonid());
            person.getTripCollection().remove(trip);
            em.remove(trip);
            et.commit();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }

    }


    /**
     * <p>Update an existing {@link Person} instance.</p>
     *
     * @param person The {@link Person} instance to be updated
     *
     * @return The updated {@link Person} instance
     */
    public Person updatePerson(Person person) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            person = em.merge(person);
            et.commit();
            return person;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }

    }


    /**
     * <p>Update an existing {@link Trip} instance.</p>
     *
     * @param trip The {@link Trip} instance to be updated
     *
     * @return The updated {@link Trip} instance
     */
    public Trip updateTrip(Trip trip) {

        EntityManager em = entityManager();
        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            trip = em.merge(trip);
            et.commit();
            return trip;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
            em.close();
        }

    }


    // --------------------------------------------------------- Support Methods


    /**
     * <p>Return an entity manager instance that may be used for a specific
     * transaction, and then closed.</p>
     */
    private EntityManager entityManager() {

        return entityManagerFactory.createEntityManager();

    }


}
